@@ -131,7 +131,6 @@
 #'  \item{_se}{Standard error}
 #' }
 #'
-#'
 #' \strong{For example:}
 #' \describe{
 #'   \item{WHZ_pop}{Weight-for-height weighted sample size}
@@ -147,27 +146,48 @@
 #'   interval limit}
 #' }
 #'
+#' \strong{Methods:}
+#'
+#' The \code{survey} package is used to compute all estimates.
+#' Confidence intervals are computed with a significance level of \code{0.05}.
+#' \link[survey]{svymean} is used the z-score mean and CIs, while for the
+#' prevalence estimates \link[survey]{svyciprop} is used with the \code{logit}
+#' method.
+#'
+#' For cases where \code{cluster/strata/sw is NULL} a fast approximation
+#' is used that does not use the \code{survey} package but computes the same
+#' results within bounds (maximum empirical difference is < 0.0001).
+#'
+#' @references
+#' T. Lumley (2024) "survey: analysis of complex survey samples".
+#' R package version 4.4.
+#'
 #' @include anthro-package.R
 #' @include assertions.R
+#' @include prevalence-simple.R
+#' @include prevalence-hybrid-sw.R
+#' @include prevalence-survey.R
 #' @importFrom stats confint
 #' @importFrom stats as.formula
 #' @importFrom stats setNames
 #' @export
-anthro_prevalence <- function(sex,
-                              age = NA_real_,
-                              is_age_in_month = FALSE,
-                              weight = NA_real_,
-                              lenhei = NA_real_,
-                              measure = NA_character_,
-                              oedema = "n",
-                              sw = NULL,
-                              cluster = NULL,
-                              strata = NULL,
-                              typeres = NA_character_,
-                              gregion = NA_character_,
-                              wealthq = NA_character_,
-                              mothered = NA_character_,
-                              othergr = NA_character_) {
+anthro_prevalence <- function(
+  sex,
+  age = NA_real_,
+  is_age_in_month = FALSE,
+  weight = NA_real_,
+  lenhei = NA_real_,
+  measure = NA_character_,
+  oedema = "n",
+  sw = NULL,
+  cluster = NULL,
+  strata = NULL,
+  typeres = NA_character_,
+  gregion = NA_character_,
+  wealthq = NA_character_,
+  mothered = NA_character_,
+  othergr = NA_character_
+) {
   # the other variables are being checked by anthro_zscores
   assert_character_or_numeric(typeres)
   assert_character_or_numeric(gregion)
@@ -215,8 +235,14 @@ anthro_prevalence <- function(sex,
   zscores_orig <- zscores
   stopifnot(c("zlen", "zwei", "zwfl", "zbmi") %in% colnames(zscores))
 
-  input[["age_in_days"]] <- age_to_days(input[["age"]], is_age_in_month = is_age_in_month)
-  input[["age_in_months"]] <- age_to_months(input[["age"]], is_age_in_month = is_age_in_month)
+  input[["age_in_days"]] <- age_to_days(
+    input[["age"]],
+    is_age_in_month = is_age_in_month
+  )
+  input[["age_in_months"]] <- age_to_months(
+    input[["age"]],
+    is_age_in_month = is_age_in_month
+  )
 
   input[["oedema"]] <- standardize_oedema_var(input[["oedema"]])
 
@@ -227,19 +253,28 @@ anthro_prevalence <- function(sex,
   # (children w/ oedema count in prevalence even if z-score missing
   # for weight related indicators)
   # this is redundant with the filter that age_in_days <= 1826
-  stopifnot(is.character(input[["oedema"]]), all(!is.na(input[["oedema"]]) &
-    oedema %in% c("n", "y")))
+  stopifnot(
+    is.character(input[["oedema"]]),
+    all(
+      !is.na(input[["oedema"]]) &
+        oedema %in% c("n", "y")
+    )
+  )
   oedema_to_n <- !is.na(input[["age_in_days"]]) & input[["age_in_days"]] > 1826
   input[["oedema"]][oedema_to_n] <- "n"
 
   # retain only those records where age <= 1826 or is.na(age)
-  filter_zscores <- is.na(input[["age_in_days"]]) | input[["age_in_days"]] <= 1826
+  filter_zscores <- is.na(input[["age_in_days"]]) |
+    input[["age_in_days"]] <= 1826
 
   # warn if rows are excluded
   no_excluded <- sum(!filter_zscores, na.rm = TRUE)
   if (no_excluded > 0L) {
     row_label <- if (no_excluded == 1L) "row" else "rows"
-    warning(no_excluded, " ", row_label,
+    warning(
+      no_excluded,
+      " ",
+      row_label,
       " will be excluded for the prevalence ",
       "computation due to missing age or age > 1826 days ",
       call. = FALSE
@@ -253,7 +288,8 @@ anthro_prevalence <- function(sex,
     input[["wealthq"]] %in% as.character(1L:5L) |
     input[["wealthq"]] %in% paste0("Q", 1L:5L)
   if (!all(valid_wealthq_values)) {
-    warning("Some entries in wealthq are out of range. Allowed values are: ",
+    warning(
+      "Some entries in wealthq are out of range. Allowed values are: ",
       "NA, 1, 2, 3, 4, 5 or Q1, Q2, Q3, Q4, Q5. ",
       "All other values will be converted to NA",
       call. = FALSE
@@ -268,17 +304,26 @@ anthro_prevalence <- function(sex,
   input[["wealthq"]][wealth_q_in("3")] <- "Q3"
   input[["wealthq"]][wealth_q_in("4")] <- "Q4"
   input[["wealthq"]][wealth_q_in("5")] <- "Q5: Richest"
-  input[["wealthq"]] <- factor(input[["wealthq"]], levels = c(
-    "Q1: Poorest", "Q2", "Q3",
-    "Q4", "Q5: Richest"
-  ))
+  input[["wealthq"]] <- factor(
+    input[["wealthq"]],
+    levels = c(
+      "Q1: Poorest",
+      "Q2",
+      "Q3",
+      "Q4",
+      "Q5: Richest"
+    )
+  )
 
   input[["sex"]] <- as.character(standardize_sex_var(input[["sex"]]))
   input[["sex"]][input[["sex"]] %in% "1"] <- "Male"
   input[["sex"]][input[["sex"]] %in% "2"] <- "Female"
   input[["sex"]] <- factor(input[["sex"]], levels = c("Female", "Male"))
 
-  input[["age_sex_interaction"]] <- interaction(input[["age_group"]], input[["sex"]])
+  input[["age_sex_interaction"]] <- interaction(
+    input[["age_group"]],
+    input[["sex"]]
+  )
 
   # here we prepare all different strata that are being computed
   optional_strata_labels <-
@@ -310,7 +355,8 @@ anthro_prevalence <- function(sex,
     logical(1L)
   )
   if (all(included_strata == FALSE)) {
-    stop("It seems that all values have been removed from the analysis, ",
+    stop(
+      "It seems that all values have been removed from the analysis, ",
       "due to specific exclusion rules. We stop the calculation here.",
       call. = FALSE
     )
@@ -362,26 +408,35 @@ anthro_prevalence <- function(sex,
     data = prev_data,
     zscores_to_compute = list(
       list(
-        name = "HA", column = "len",
-        with_cutoffs = TRUE, with_auxiliary_zscore_column = FALSE
+        name = "HA",
+        column = "len",
+        with_cutoffs = TRUE,
+        with_auxiliary_zscore_column = FALSE
       ),
       list(
-        name = "WA", column = "wei",
-        with_cutoffs = TRUE, with_auxiliary_zscore_column = TRUE
+        name = "WA",
+        column = "wei",
+        with_cutoffs = TRUE,
+        with_auxiliary_zscore_column = TRUE
       ),
       list(
-        name = "BMI", column = "bmi",
-        with_cutoffs = TRUE, with_auxiliary_zscore_column = TRUE
+        name = "BMI",
+        column = "bmi",
+        with_cutoffs = TRUE,
+        with_auxiliary_zscore_column = TRUE
       ),
       list(
-        name = "WH", column = "wfl",
-        with_cutoffs = TRUE, with_auxiliary_zscore_column = TRUE
+        name = "WH",
+        column = "wfl",
+        with_cutoffs = TRUE,
+        with_auxiliary_zscore_column = TRUE
       )
     ),
     survey_subsets = setNames(strata_cols[included_strata], strata_label)
   )
 }
 
+prevalence_significance_level <- 0.05
 
 #' @importFrom survey svytotal
 #' @importFrom survey svyby
@@ -389,22 +444,32 @@ anthro_prevalence <- function(sex,
 #' @importFrom survey degf
 #' @importFrom survey svymean
 #' @importFrom survey svyvar
-compute_prevalence_of_zscores <- function(data,
-                                          zscores_to_compute,
-                                          survey_subsets) {
+compute_prevalence_of_zscores <- function(
+  data,
+  zscores_to_compute,
+  survey_subsets
+) {
   stopifnot(
     is.data.frame(data),
     all(c("oedema") %in% colnames(data)),
     is.list(zscores_to_compute),
-    all(vapply(zscores_to_compute, function(x) {
-      is.list(x)
-    }, logical(1L))),
-    is.null(data[["sampling_weights"]]) || is.numeric(data[["sampling_weights"]]),
+    all(vapply(
+      zscores_to_compute,
+      function(x) {
+        is.list(x)
+      },
+      logical(1L)
+    )),
+    is.null(data[["sampling_weights"]]) ||
+      is.numeric(data[["sampling_weights"]]),
     is.list(survey_subsets),
     is.character(names(survey_subsets))
   )
 
-  if (!is.null(data[["sampling_weights"]]) && any(data[["sampling_weights"]] < 0, na.rm = TRUE)) {
+  if (
+    !is.null(data[["sampling_weights"]]) &&
+      any(data[["sampling_weights"]] < 0, na.rm = TRUE)
+  ) {
     stop(
       "Negative sampling weights are not allowed.",
       call. = FALSE
@@ -413,7 +478,9 @@ compute_prevalence_of_zscores <- function(data,
 
   zscores_to_compute <- lapply(zscores_to_compute, function(indicator) {
     if (isTRUE(indicator$with_cutoffs)) {
-      indicator$cutoffs <- generate_cutoffs(prev_prevalence_column_name(indicator))
+      indicator$cutoffs <- generate_cutoffs(prev_prevalence_column_name(
+        indicator
+      ))
     }
     indicator
   })
@@ -434,7 +501,10 @@ compute_prevalence_of_zscores <- function(data,
   no_excluded <- old_data_nrows - nrow(data)
   if (no_excluded > 0L) {
     row_label <- if (no_excluded == 1L) "row" else "rows"
-    warning(no_excluded, " ", row_label,
+    warning(
+      no_excluded,
+      " ",
+      row_label,
       " will be excluded for the prevalence ",
       "computation due to missing cluster and strata.",
       call. = FALSE
@@ -442,15 +512,21 @@ compute_prevalence_of_zscores <- function(data,
   }
 
   # set all sw that are NA to 0
-  if (!is.null(data[["sampling_weights"]]) &&
-    anyNA(data[["sampling_weights"]])) {
+  if (
+    !is.null(data[["sampling_weights"]]) &&
+      anyNA(data[["sampling_weights"]])
+  ) {
     sw <- data[["sampling_weights"]]
     na_sw <- is.na(sw)
     sw[na_sw] <- 0
     data[["sampling_weights"]] <- sw
     no_excluded <- sum(na_sw)
     row_label <- if (no_excluded == 1L) "row" else "rows"
-    warning(no_excluded, " ", row_label, " had missing sampling weights",
+    warning(
+      no_excluded,
+      " ",
+      row_label,
+      " had missing sampling weights",
       " and they will be set to 0. ",
       "Doing this excludes them in the prevalence calculation.",
       call. = FALSE
@@ -460,7 +536,8 @@ compute_prevalence_of_zscores <- function(data,
   res <- set_flagged_zscores_to_NA(zscores_to_compute, data)
   res <- create_zscore_auxiliary_columns(zscores_to_compute, res)
   res <- create_cutoff_columns_for_each_zscore(zscores_to_compute, res)
-  survey_design <- build_survey_design(res)
+
+  survey_design <- build_design(res)
 
   final_result <- compute_prevalence_for_each_subset_and_indicator(
     subset_col_names = survey_subsets,
@@ -486,46 +563,37 @@ generate_cutoffs <- function(data_column) {
   })
 }
 
-#' @importFrom survey svydesign
-build_survey_design <- function(survey_data) {
-  has_cluster_info <- !is.null(survey_data[["cluster"]])
-  has_strata_info <- !is.null(survey_data[["strata"]])
-  has_sampling_weights <- !is.null(survey_data[["sampling_weights"]])
-  just_one_cluster <- has_cluster_info && length(unique(survey_data[["cluster"]])) == 1L
-  any_strata_na <- anyNA(survey_data[["strata"]])
-  stopifnot(!any_strata_na)
-  cluster_formula <- if (just_one_cluster || !has_cluster_info) {
-    ~1
-  } else {
-    ~cluster
+#' build_design constructs a custom design object that is used to compute
+#' all indicators.
+#' The function decides if the survey package is used for all indicator
+#' or if a specialized method can be used to compute the results faster. For
+#' example when there are no cluster/strata and sampling weights.
+#' @noRd
+build_design <- function(survey_data) {
+  # we can override the choice here using internal options.
+  # these options are not part of the public api and can change at any time.
+  if (isTRUE(getOption("anthro.internal.compute_method") == "survey")) {
+    return(build_survey_design(survey_data))
   }
-  strata_formula <- if (has_strata_info) {
-    ~strata
+  is_simple_problem <- is.null(survey_data[["cluster"]]) &&
+    is.null(survey_data[["strata"]]) &&
+    (is.null(survey_data[["sampling_weights"]]) ||
+      all(survey_data[["sampling_weights"]] == 1, na.rm = TRUE))
+  if (is_simple_problem) {
+    return(build_simple_design(survey_data))
   }
-  sampling_weights_formula <- if (has_sampling_weights) {
-    ~sampling_weights
+  is_simple_problem_with_sw <- is.null(survey_data[["cluster"]]) &&
+    is.null(survey_data[["strata"]])
+  if (is_simple_problem_with_sw) {
+    return(build_hybrid_design(survey_data))
   }
-  design <- svydesign(
-    ids = cluster_formula,
-    strata = strata_formula,
-    weights = sampling_weights_formula,
-    data = survey_data,
-    nest = TRUE
-  )
-  design_unweighted <- svydesign(
-    ids = cluster_formula,
-    strata = strata_formula,
-    weights = NULL,
-    data = survey_data,
-    nest = TRUE
-  )
-  list(
-    design = design,
-    design_unweighted = design_unweighted
-  )
+  build_survey_design(survey_data)
 }
 
-compute_prevalence_stunting_wasting <- function(survey_design, subset_col_name) {
+compute_prevalence_stunting_wasting <- function(
+  survey_design,
+  subset_col_name
+) {
   indicator <- list(
     name = "HA_2_WH_2",
     column = "len_zwfl_aux_stunting_wasting",
@@ -535,13 +603,18 @@ compute_prevalence_stunting_wasting <- function(survey_design, subset_col_name)
   list(
     compute_prevalence_sample_size(survey_design, indicator, subset_col_name),
     compute_prevalence_estimates_for_column(
-      survey_design$design, indicator$name,
-      subset_col_name, prev_prevalence_column_name(indicator)
+      survey_design,
+      indicator$name,
+      subset_col_name,
+      prev_prevalence_column_name(indicator)
     )
   )
 }
 
-compute_prevalence_stunting_overweight <- function(survey_design, subset_col_name) {
+compute_prevalence_stunting_overweight <- function(
+  survey_design,
+  subset_col_name
+) {
   indicator <- list(
     name = "HA_2_WH2",
     column = "len_zwfl_aux_stunting_overweight",
@@ -551,67 +624,88 @@ compute_prevalence_stunting_overweight <- function(survey_design, subset_col_nam
   list(
     compute_prevalence_sample_size(survey_design, indicator, subset_col_name),
     compute_prevalence_estimates_for_column(
-      survey_design$design, indicator$name,
-      subset_col_name, prev_prevalence_column_name(indicator)
+      survey_design,
+      indicator$name,
+      subset_col_name,
+      prev_prevalence_column_name(indicator)
     )
   )
 }
 
-compute_prevalence_for_each_subset_and_indicator <- function(subset_col_names,
-                                                             subset_labels,
-                                                             survey_design,
-                                                             indicators) {
+compute_prevalence_for_each_subset_and_indicator <- function(
+  subset_col_names,
+  subset_labels,
+  survey_design,
+  indicators
+) {
   has_zlen_and_wfl <- all(
-    c("zlen", "zwfl_aux") %in% colnames(survey_design$design$variables)
+    c("zlen", "zwfl_aux") %in% column_names(survey_design)
   )
   # This double apply could be moved down to the survey package as it
   # also supports calculating multiple values at once.
-  subset_results <- mapply(function(subset_col_name, label) {
-    indicator_results <- lapply(indicators, function(indicator) {
-      # some svyby calls produce glm.fit warnings.
-      # It was decided to suppress these warnings.
-      suppressWarnings({
-        c(
-          list(compute_prevalence_sample_size(
-            survey_design, indicator, subset_col_name
-          )),
-          compute_prevalence_cutoff_summaries(
-            survey_design$design, indicator, subset_col_name
-          ),
-          list(compute_prevalence_zscore_summaries(
-            survey_design$design, indicator, subset_col_name
-          ))
-        )
+  subset_results <- mapply(
+    function(subset_col_name, label) {
+      indicator_results <- lapply(indicators, function(indicator) {
+        # some svyby calls produce glm.fit warnings.
+        # It was decided to suppress these warnings.
+        suppressWarnings({
+          c(
+            list(compute_prevalence_sample_size(
+              survey_design,
+              indicator,
+              subset_col_name
+            )),
+            compute_prevalence_cutoff_summaries(
+              survey_design,
+              indicator,
+              subset_col_name
+            ),
+            list(compute_prevalence_zscore_summaries(
+              survey_design,
+              indicator,
+              subset_col_name
+            ))
+          )
+        })
       })
-    })
-    indicator_results <- unlist(indicator_results, recursive = FALSE) # flatten
+      indicator_results <- unlist(indicator_results, recursive = FALSE) # flatten
 
-    suppressWarnings({
-      if (has_zlen_and_wfl) {
-        indicator_results <- c(
-          indicator_results,
-          compute_prevalence_stunting_wasting(survey_design, subset_col_name),
-          compute_prevalence_stunting_overweight(survey_design, subset_col_name)
-        )
+      suppressWarnings({
+        if (has_zlen_and_wfl) {
+          indicator_results <- c(
+            indicator_results,
+            compute_prevalence_stunting_wasting(survey_design, subset_col_name),
+            compute_prevalence_stunting_overweight(
+              survey_design,
+              subset_col_name
+            )
+          )
+        }
+      })
+
+      # now we merge everything into one df
+      # we assume the first indicator results DF has all levels of the Group
+      # then we merge everything and after that we have to restore the original
+      # ordering of the levels (they get lost by the `merge` calls (sometimes))
+      original_group_ordering <- indicator_results[[1]][["Group"]]
+      res <- Reduce(
+        function(acc, el) {
+          merge(acc, el, by = "Group", sort = FALSE, all = TRUE)
+        },
+        indicator_results
+      )
+      stopifnot(setequal(res[["Group"]], original_group_ordering))
+      res <- res[match(original_group_ordering, res[["Group"]]), , drop = FALSE]
+
+      if (label != "All") {
+        res[["Group"]] <- paste0(label, ": ", res[["Group"]])
       }
-    })
-
-    # now we merge everything into one df
-    # we assume the first indicator results DF has all levels of the Group
-    # then we merge everything and after that we have to restore the original
-    # ordering of the levels (they get lost by the `merge` calls (sometimes))
-    original_group_ordering <- indicator_results[[1]][["Group"]]
-    res <- Reduce(function(acc, el) {
-      merge(acc, el, by = "Group", sort = FALSE, all = TRUE)
-    }, indicator_results)
-    stopifnot(setequal(res[["Group"]], original_group_ordering))
-    res <- res[match(original_group_ordering, res[["Group"]]), , drop = FALSE]
-
-    if (label != "All") {
-      res[["Group"]] <- paste0(label, ": ", res[["Group"]])
-    }
-    res
-  }, subset_col_names, subset_labels, SIMPLIFY = FALSE)
+      res
+    },
+    subset_col_names,
+    subset_labels,
+    SIMPLIFY = FALSE
+  )
   res <- do.call(rbind, subset_results)
   rownames(res) <- NULL
   res
@@ -655,17 +749,15 @@ create_zscore_auxiliary_columns <- function(zscores_to_compute, dataframe) {
   }
   has_zlen_and_wfl <- all(c("zlen", "zwfl_aux") %in% colnames(dataframe))
   if (has_zlen_and_wfl) {
+    zlen <- dataframe[["zlen"]]
+    zwfl_aux <- dataframe[["zwfl_aux"]]
+    is_na <- is.na(zlen) | is.na(zwfl_aux)
+    res <- rep.int(0L, length(zlen))
+    res[is_na] <- NA_integer_
     make_combined_aux_col <- function(flag_fun) {
-      as.integer(
-        with(
-          dataframe,
-          ifelse(
-            is.na(zlen) | is.na(zwfl_aux),
-            NA_integer_,
-            ifelse(zlen < -2 & flag_fun(zwfl_aux), 1L, 0L)
-          )
-        )
-      )
+      res_local <- res
+      res_local[zlen < -2 & flag_fun(zwfl_aux)] <- 1L
+      res_local
     }
     dataframe[["zlen_zwfl_aux_stunting_wasting"]] <- make_combined_aux_col(
       flag_fun = function(x) x < -2
@@ -677,6 +769,7 @@ create_zscore_auxiliary_columns <- function(zscores_to_compute, dataframe) {
   dataframe
 }
 
+
 set_flagged_zscores_to_NA <- function(indicators, dataframe) {
   for (indicator in indicators) {
     zscore_col <- prev_zscore_value_column(indicator)
@@ -702,44 +795,20 @@ create_cutoff_columns_for_each_zscore <- function(indicators, dataframe) {
   dataframe
 }
 
-compute_prevalence_sample_size <- function(survey_design, indicator, subset_col_name) {
-  indicator_name <- indicator$name
-  expr_name <- paste0("I(!is.na(", prev_prevalence_column_name(indicator), "))")
-  prev_formula <- as.formula(paste0("~", expr_name))
-  subset_formula <- as.formula(paste0("~", subset_col_name))
-  prev <- svyby(
-    prev_formula,
-    subset_formula,
-    survey_design$design,
-    svytotal,
-    drop.empty.groups = FALSE
-  )
-  unweighted_prev <- svyby(
-    prev_formula,
-    subset_formula,
-    survey_design$design_unweighted,
-    svytotal,
-    drop.empty.groups = FALSE
-  )
-  pop_weighted <- prev[[paste0(expr_name, "TRUE")]]
-  pop_unweighted <- unweighted_prev[[paste0(expr_name, "TRUE")]]
-
-  # syby returns NA for empty levels. We set the count to 0 for these.
-  pop_weighted[is.na(pop_weighted)] <- 0
-  pop_unweighted[is.na(pop_unweighted)] <- 0
-
-  stopifnot( # check that subsets come in the right order
-    all(prev[[subset_col_name]] == unweighted_prev[[subset_col_name]])
-  )
-  res <- data.frame(
-    Group = as.character(prev[[subset_col_name]]),
-    pop = pop_weighted,
-    unwpop = pop_unweighted,
-    stringsAsFactors = FALSE
+compute_prevalence_sample_size <- function(
+  survey_design,
+  indicator,
+  subset_col_name
+) {
+  res <- compute_prevalence_sample_size_by(
+    survey_design,
+    indicator,
+    subset_col_name
   )
 
   # due to backwards compatibility, columns HA_2_WH_2 and HA_2_WH2 will not
   # have a suffix Z for now
+  indicator_name <- indicator$name
   suffix <- if (indicator_name %in% c("HA_2_WH_2", "HA_2_WH2")) {
     c("_pop", "_unwpop")
   } else {
@@ -748,32 +817,41 @@ compute_prevalence_sample_size <- function(survey_design, indicator, subset_col_
   colnames(res) <- c("Group", paste0(indicator_name, suffix))
   # only HA_2_WH_2 has pop and unwpop columns, but not HA_2_WH2
   if (indicator_name == "HA_2_WH2") {
-    col_idx_to_drop <- which(colnames(res) %in% c("HA_2_WH2_pop", "HA_2_WH2_unwpop"))
+    col_idx_to_drop <- which(
+      colnames(res) %in% c("HA_2_WH2_pop", "HA_2_WH2_unwpop")
+    )
     res <- res[, colnames(res)[-col_idx_to_drop], drop = FALSE]
   }
   res
 }
 
-compute_prevalence_cutoff_summaries <- function(survey_design, indicator, subset_col_name) {
+compute_prevalence_cutoff_summaries <- function(
+  survey_design,
+  indicator,
+  subset_col_name
+) {
   indicator_name <- indicator$name
   lapply(indicator$cutoffs, function(cutoff) {
     prev_col_name <- paste0("prev_", indicator_name, cutoff$suffix)
     compute_prevalence_estimates_for_column(
       survey_design,
       indicator_name = paste0(indicator_name, cutoff$suffix),
-      subset_col_name, prev_col_name
+      subset_col_name,
+      prev_col_name
     )
   })
 }
 
-compute_prevalence_estimates_for_column <- function(survey_design, indicator_name, subset_col_name, prev_col_name) {
-  subset_formula <- as.formula(paste0("~", subset_col_name))
-  prev_col_formula <- as.formula(paste0("~", prev_col_name))
-  all_na <- all(is.na(survey_design$variables[[prev_col_name]]))
-
+compute_prevalence_estimates_for_column <- function(
+  survey_design,
+  indicator_name,
+  subset_col_name,
+  prev_col_name
+) {
+  all_na <- all(is.na(column_values(survey_design, prev_col_name)))
   res <- if (all_na) {
     data.frame(
-      Group = as.character(unique(survey_design$variables[[subset_col_name]])),
+      Group = unique_groups(column_values(survey_design, subset_col_name)),
       r = NA_real_,
       se = NA_real_,
       ll = NA_real_,
@@ -781,55 +859,33 @@ compute_prevalence_estimates_for_column <- function(survey_design, indicator_nam
       stringsAsFactors = FALSE
     )
   } else {
-    mean_est_prev <- svyby(
-      prev_col_formula,
-      subset_formula,
+    compute_prevalence_estimates_for_column_by(
       survey_design,
-      svymean,
-      na.rm = TRUE,
-      na.rm.all = TRUE,
-      drop.empty.groups = FALSE
-    )
-
-    mean_est_ci_prev <- svyby(
-      prev_col_formula,
-      subset_formula,
-      survey_design,
-      svyciprop,
-      vartype = "ci",
-      df = degf(survey_design),
-      method = "logit",
-      drop.empty.groups = FALSE,
-      na.rm.all = TRUE,
-      level = 0.95
-    )[, 3L:4L]
-    data.frame(
-      Group = as.character(mean_est_prev[[subset_col_name]]),
-      r = mean_est_prev[[prev_col_name]] * 100,
-      se = survey::SE(mean_est_prev) * 100,
-      ll = mean_est_ci_prev$ci_l * 100,
-      ul = mean_est_ci_prev$ci_u * 100,
-      stringsAsFactors = FALSE
+      indicator_name,
+      subset_col_name,
+      prev_col_name
     )
   }
   value_col_names <- paste0(
-    indicator_name, c("_r", "_se", "_ll", "_ul")
+    indicator_name,
+    c("_r", "_se", "_ll", "_ul")
   )
   col_names <- c("Group", value_col_names)
   colnames(res) <- col_names
   res
 }
 
-compute_prevalence_zscore_summaries <- function(survey_design,
-                                                indicator, subset_col_name) {
-  indicator_name <- indicator$name
+
+compute_prevalence_zscore_summaries <- function(
+  survey_design,
+  indicator,
+  subset_col_name
+) {
   zscore_col_name <- prev_zscore_value_column(indicator)
-  zscore_formula <- as.formula(paste0("~", zscore_col_name))
-  subset_formula <- as.formula(paste0("~", subset_col_name))
-  all_na <- all(is.na(survey_design$variables[[zscore_col_name]]))
+  all_na <- all(is.na(column_values(survey_design, zscore_col_name)))
   res <- if (all_na) {
     data.frame(
-      Group = as.character(unique(survey_design$variables[[subset_col_name]])),
+      Group = unique_groups(column_values(survey_design, subset_col_name)),
       r = NA_real_,
       se = NA_real_,
       ll = NA_real_,
@@ -838,60 +894,92 @@ compute_prevalence_zscore_summaries <- function(survey_design,
       stringsAsFactors = FALSE
     )
   } else {
-    mean_est_summary <- svyby(
-      zscore_formula,
-      subset_formula,
+    compute_prevalence_zscore_summaries_by(
       survey_design,
-      svymean,
-      na.rm = TRUE,
-      na.rm.all = TRUE,
-      drop.empty.groups = FALSE
-    )
-    mean_est_ci_summary <- confint(
-      mean_est_summary,
-      df = degf(survey_design),
-      level = 0.95
-    )
-
-    # when there is only one observation in the subset, svyvar returns a
-    # length 2 object which breaks svyby. We thus provide a robust
-    # version of svyvar that returns NA if a length 2 object is returned.
-    # this will be fixed in survey >= 4.2
-    robust_svyvar <- function(x, design, na.rm = FALSE, ...) {
-      res <- svyvar(x, design, na.rm = na.rm, ...)
-      if (length(res) == 2) {
-        new_res <- NA_real_
-        attr(new_res, "names") <- prev_zscore_value_column(indicator)
-        attr(new_res, "var") <- NA_real_
-        attr(new_res, "statistic") <- "variance"
-        class(new_res) <- c("svyvar", "svystat", "numeric")
-        return(new_res)
-      }
-      res
-    }
-    mean_est_sd_summary <- svyby(
-      zscore_formula,
-      subset_formula,
-      survey_design,
-      robust_svyvar,
-      na.rm = TRUE,
-      na.rm.all = TRUE,
-      drop.empty.groups = FALSE
-    )
-    data.frame(
-      Group = as.character(mean_est_summary[[subset_col_name]]),
-      r = mean_est_summary[[prev_zscore_value_column(indicator)]],
-      se = survey::SE(mean_est_summary),
-      ll = mean_est_ci_summary[, 1L, drop = TRUE],
-      ul = mean_est_ci_summary[, 2L, drop = TRUE],
-      stdev = sqrt(mean_est_sd_summary[, 2L, drop = TRUE]),
-      stringsAsFactors = FALSE
+      indicator,
+      subset_col_name
     )
   }
 
+  indicator_name <- indicator$name
   value_col_names <- paste0(
-    indicator_name, c("_r", "_se", "_ll", "_ul", "_stdev")
+    indicator_name,
+    c("_r", "_se", "_ll", "_ul", "_stdev")
   )
   colnames(res) <- c("Group", value_col_names)
   res
 }
+
+unique_groups <- function(values) {
+  as.character(unique(values[!is.na(values)]))
+}
+
+
+build_simple_design <- function(survey_data) {
+  structure(
+    list(data = survey_data),
+    class = "simple_design"
+  )
+}
+
+build_hybrid_design <- function(survey_data) {
+  # for cases with sw > 0 but cluster/strata = NULL, we just need
+  # a single survey design object
+  design <- svydesign(
+    ids = ~1,
+    strata = NULL,
+    weights = ~sampling_weights,
+    data = survey_data,
+    nest = TRUE
+  )
+  structure(
+    list(
+      data = survey_data,
+      design = design
+    ),
+    class = "hybrid_design"
+  )
+}
+
+#' @importFrom survey svydesign
+build_survey_design <- function(survey_data) {
+  has_cluster_info <- !is.null(survey_data[["cluster"]])
+  has_strata_info <- !is.null(survey_data[["strata"]])
+  has_sampling_weights <- !is.null(survey_data[["sampling_weights"]])
+  just_one_cluster <- has_cluster_info &&
+    length(unique(survey_data[["cluster"]])) == 1L
+  any_strata_na <- anyNA(survey_data[["strata"]])
+  stopifnot(!any_strata_na)
+  cluster_formula <- if (just_one_cluster || !has_cluster_info) {
+    ~1
+  } else {
+    ~cluster
+  }
+  strata_formula <- if (has_strata_info) {
+    ~strata
+  }
+  sampling_weights_formula <- if (has_sampling_weights) {
+    ~sampling_weights
+  }
+  design <- svydesign(
+    ids = cluster_formula,
+    strata = strata_formula,
+    weights = sampling_weights_formula,
+    data = survey_data,
+    nest = TRUE
+  )
+  design_unweighted <- svydesign(
+    ids = cluster_formula,
+    strata = strata_formula,
+    weights = NULL,
+    data = survey_data,
+    nest = TRUE
+  )
+  structure(
+    list(
+      design = design,
+      design_unweighted = design_unweighted
+    ),
+    class = "survey_design"
+  )
+}
