@@ -11,8 +11,14 @@
 #' @include z-score-helper.R
 #' @noRd
 anthro_zscore_subscapular_skinfold_for_age <-
-  function(subskin, age_in_days, age_in_months, sex, flag_threshold = 5,
-           growthstandards = growthstandards_ssanthro) {
+  function(
+    subskin,
+    age_in_days,
+    age_in_months,
+    sex,
+    flag_threshold = 5,
+    growthstandards = growthstandards_ssanthro
+  ) {
     anthro_zscore_adjusted(
       name = "ss",
       measure = subskin,
